aboutsummaryrefslogtreecommitdiff
path: root/libdiskfs
diff options
context:
space:
mode:
Diffstat (limited to 'libdiskfs')
-rw-r--r--libdiskfs/Makefile2
-rw-r--r--libdiskfs/file-get-children.c96
-rw-r--r--libdiskfs/fsys-get-children.c74
-rw-r--r--libdiskfs/fsys-get-source.c (renamed from libdiskfs/file-get-source.c)26
4 files changed, 89 insertions, 109 deletions
diff --git a/libdiskfs/Makefile b/libdiskfs/Makefile
index 93c96fa6..5716ac5b 100644
--- a/libdiskfs/Makefile
+++ b/libdiskfs/Makefile
@@ -35,7 +35,7 @@ IOSRCS= io-async-icky.c io-async.c io-duplicate.c io-get-conch.c io-revoke.c \
io-select.c io-stat.c io-stubs.c io-write.c io-version.c io-sigio.c
FSYSSRCS=fsys-getroot.c fsys-goaway.c fsys-startup.c fsys-getfile.c \
fsys-options.c fsys-syncfs.c fsys-forward.c \
- file-get-children.c file-get-source.c
+ fsys-get-children.c fsys-get-source.c
IFSOCKSRCS=ifsock.c
OTHERSRCS = conch-fetch.c conch-set.c dir-clear.c dir-init.c dir-renamed.c \
extern-inline.c \
diff --git a/libdiskfs/file-get-children.c b/libdiskfs/file-get-children.c
deleted file mode 100644
index 98d5d60e..00000000
--- a/libdiskfs/file-get-children.c
+++ /dev/null
@@ -1,96 +0,0 @@
-/* file_get_children
-
- Copyright (C) 2013 Free Software Foundation, Inc.
-
- Written by Justus Winter <4winter@informatik.uni-hamburg.de>
-
- This file is part of the GNU Hurd.
-
- The GNU Hurd is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
- any later version.
-
- The GNU Hurd is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with the GNU Hurd. If not, see <http://www.gnu.org/licenses/>. */
-
-#include "priv.h"
-#include "fs_S.h"
-
-#include <argz.h>
-
-/* Return any active translators bound to nodes below CRED. CHILDREN
- is an argz vector containing file names relative to the path of
- CRED. */
-error_t
-diskfs_S_file_get_children (struct protid *cred,
- char **children,
- mach_msg_type_number_t *children_len)
-{
- error_t err;
- if (! cred
- || cred->pi.bucket != diskfs_port_bucket
- || cred->pi.class != diskfs_protid_class)
- return EOPNOTSUPP;
-
- /* check_access performs the same permission check as is normally
- done, i.e. it checks that all but the last path components are
- executable by the requesting user and that the last component is
- readable. */
- error_t check_access (const char *path)
- {
- error_t err;
- char *elements = NULL;
- size_t elements_len = 0;
-
- err = argz_create_sep (path, '/', &elements, &elements_len);
- if (err)
- return err;
-
- struct node *dp = diskfs_root_node;
-
- for (char *entry = elements;
- entry;
- entry = argz_next (elements, elements_len, entry))
- {
- struct node *next;
- err = diskfs_lookup (dp, entry, LOOKUP, &next, NULL, cred);
-
- if (dp != diskfs_root_node)
- diskfs_nput (dp);
-
- if (err)
- return err;
-
- dp = next;
- }
-
- err = fshelp_access (&dp->dn_stat, S_IRUSR, cred->user);
- diskfs_nput (dp);
- return err;
- }
-
-
- char *c = NULL;
- size_t c_len = 0;
-
- err = fshelp_get_active_translators (&c, &c_len, check_access,
- cred->po->path);
- if (err)
- goto errout;
-
- err = iohelp_return_malloced_buffer (c, c_len, children, children_len);
- if (err)
- goto errout;
-
- c = NULL; /* c was freed by iohelp_return_malloced_buffer. */
-
- errout:
- free (c);
- return err;
-}
diff --git a/libdiskfs/fsys-get-children.c b/libdiskfs/fsys-get-children.c
new file mode 100644
index 00000000..7e2a37b1
--- /dev/null
+++ b/libdiskfs/fsys-get-children.c
@@ -0,0 +1,74 @@
+/* fsys_get_children
+
+ Copyright (C) 2017 Free Software Foundation, Inc.
+
+ This file is part of the GNU Hurd.
+
+ The GNU Hurd is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 2, or (at your option)
+ any later version.
+
+ The GNU Hurd is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with the GNU Hurd. If not, see <http://www.gnu.org/licenses/>. */
+
+#include "priv.h"
+#include "fsys_S.h"
+
+#include <argz.h>
+
+/* Return any active child translators. NAMES is an argz vector
+ containing file names relative to the root of the translator.
+ CONTROLS is an array containing the corresponding control ports.
+ Note that translators are bound to nodes, and nodes can have zero
+ or more links in the file system, therefore there is no guarantee
+ that a translators name refers to an existing link in the file
+ system. */
+error_t
+diskfs_S_fsys_get_children (struct diskfs_control *fsys,
+ mach_port_t reply,
+ mach_msg_type_name_t replytype,
+ char **names,
+ mach_msg_type_number_t *names_len,
+ mach_port_t **controls,
+ mach_msg_type_name_t *controlsPoly,
+ mach_msg_type_number_t *controlsCnt)
+{
+ error_t err;
+ char *n = NULL;
+ size_t n_len = 0;
+ mach_port_t *c;
+ size_t c_count;
+
+ if (! fsys)
+ return EOPNOTSUPP;
+
+
+ err = fshelp_get_active_translators (&n, &n_len, &c, &c_count);
+ if (err)
+ goto errout;
+
+ err = iohelp_return_malloced_buffer (n, n_len, names, names_len);
+ if (err)
+ goto errout;
+ n = NULL; /* n was freed by iohelp_return_malloced_buffer. */
+
+ err = iohelp_return_malloced_buffer ((char *) c, c_count * sizeof *c,
+ (char **) controls, controlsCnt);
+ if (err)
+ goto errout;
+ c = NULL; /* c was freed by iohelp_return_malloced_buffer. */
+
+ *controlsPoly = MACH_MSG_TYPE_MOVE_SEND;
+ *controlsCnt = c_count;
+
+ errout:
+ free (n);
+ free (c);
+ return err;
+}
diff --git a/libdiskfs/file-get-source.c b/libdiskfs/fsys-get-source.c
index d983a826..59eee55e 100644
--- a/libdiskfs/file-get-source.c
+++ b/libdiskfs/fsys-get-source.c
@@ -1,8 +1,6 @@
-/* file_get_source
+/* fsys_get_source
- Copyright (C) 2013 Free Software Foundation, Inc.
-
- Written by Justus Winter <4winter@informatik.uni-hamburg.de>
+ Copyright (C) 2017 Free Software Foundation, Inc.
This file is part of the GNU Hurd.
@@ -20,17 +18,21 @@
along with the GNU Hurd. If not, see <http://www.gnu.org/licenses/>. */
#include "priv.h"
-#include "fs_S.h"
-
-/* Return information about the source of the receiving
- filesystem. */
+#include "fsys_S.h"
+
+/* Return information about the source of the translator. If the
+ concept of a source is applicable, SOURCE should refer to the
+ source of the translator and should be a description considered
+ appropriate in the context of the translator. For example, if the
+ translator is a filesystem residing on a block device, then SOURCE
+ should be the file name of the underlying block device. */
error_t
-diskfs_S_file_get_source (struct protid *cred,
+diskfs_S_fsys_get_source (struct diskfs_control *fsys,
+ mach_port_t reply,
+ mach_msg_type_name_t replytype,
char *source)
{
- if (! cred
- || cred->pi.bucket != diskfs_port_bucket
- || cred->pi.class != diskfs_protid_class)
+ if (! fsys)
return EOPNOTSUPP;
return diskfs_get_source (source, 1024 /* XXX */);